import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.math.*;


public class ControlCanvas extends ScaleCanvas implements MouseListener,MouseMotionListener {
    Manager M;
    SelectColor CS;
    SelectInteger S;
    EnhancedSlider STEP;
    ControlPanel ACTION,PROJ,GEOM,SYMM,VIEW;
    ControlPanelColor DISPLAY,DISPLAY2;
    PopupManager POP;
    ListenSquare INFO,INFO2,GO,STOP;
    Lever YIN,SPEED;
    Tracer TR;

     public ControlCanvas() {
	 addMouseListener(this);
	 addMouseMotionListener(this);
	 CS=new SelectColor(M,0,211,594,50);
	 setSliders();
	 setPanels();
	 YIN=new Lever(200,135,2,9);
	 SPEED=new Lever(460,80,2,10);
 	 INFO2=new ListenSquare(568,60,12,12);
	 GO=new ListenSquare(460,40,40,20);
	 STOP=new ListenSquare(510,40,40,20);
	 TR=new Tracer();
    }


    public void setSliders() {
	STEP=new EnhancedSlider(0,0,500,30,200,new Color(230,0,150),"integration step");
	STEP.POW.val=-10;
	INFO=new ListenSquare(575,2,12,12);
    }


    public void setPanels() {

       Color[] C0={new Color(80,120,255),
                   Color.white,
                   Color.white,
                   Color.black,
                   Color.white};

        String[] SymmString={"no",
                              "yes",
                              "symmetry"};
        int[] SymmState={1,0};
        SYMM=new ControlPanel(C0,SymmString,SymmState,2);
	SYMM.mode=0;


	String[] DisplayString={"backg",
                                "Sol polar",
                                "Sol long",
				"Sol latt",
				"inner bdy",
				"outer bdy",
				"geodesic 1",
				"geodesic 2",
				"cut locus",
				"curve fit",
                                "SOL display"};
	Color[] DisplayColor={new Color(0,0,0),
                              new Color(50,110,255),
			      new Color(0,200,220),
			      new Color(0,140,0),
			      new Color(255,130,0),
			      new Color(255,0,0),
                              new Color(255,255,0),
                              new Color(0,255,255),
			      new Color(255,200,0),
			      new Color(0,255,255)};

	int[] DisplayState={1,1,0,0,1,1,0,0,0,0};
        DISPLAY=new ControlPanelColor(C0,DisplayString,DisplayState,10,DisplayColor);


       String[] ActionString={"none",
                              "set Z=0",
                              "go to critical",
                              "length=critical",
                              "length=period",
			       "LIE coord locks"};
       int[] ActionState={1,0,0,0,0};
        ACTION=new ControlPanel(C0,ActionString,ActionState,5);
	ACTION.C[0]=new Color(0,200,200);
	ACTION.mode=0;

	String[] ViewString={"R3 -> SOL",
                             "SOL -> R3",
			     "view locks"};
	int[] ViewState={1,1};
        VIEW=new ControlPanel(C0,ViewString,ViewState,2);
	VIEW.C[0]=new Color(200,0,200);
	VIEW.mode=0;



	String[] Display2String={"backg",
				"current flowline",
				"current level",
				 "yinyang curve",
                                "LIE display"};
	Color[] Display2Color={new Color(0,0,0),
			       new Color(255,160,0),
			       new Color(0,200,255),
			       Color.yellow,
			       new Color(255,100,255)};

	int[] Display2State={1,0,0,0};
        DISPLAY2=new ControlPanelColor(C0,Display2String,Display2State,4,Display2Color);
	DISPLAY2.C[0]=new Color(0,200,200);


	
    }

    

    public void paint(Graphics g2) {
      Graphics2D g=(Graphics2D) g2;
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
      drawBG(g);
      drawControls(g);
   }



    public void drawBG(Graphics2D g) {
	g.setColor(new Color(20,30,50));
      g.fillRect(0,0,getWidth(),getHeight()); 
    }
    
    public void drawControls(Graphics2D g) {
	DISPLAY.render(g,0,35,100);
	SYMM.render(g,105,35,85);
	DISPLAY2.render(g,195,35,120);
	ACTION.render(g,320,35,100);
	VIEW.render2(g,105,90,85);
        STEP.render(g);
        CS.render(g);
	POP.render(g);
	INFO.infoRender(g);
	INFO2.infoRender(g);
	drawTracer(g);

	if(DISPLAY2.L[3].on==1) {
	   YIN.render2(g,"yinyang curve detail",new Color(0,200,200));
	}
    }


    public void drawTracer(Graphics2D g) {
	SPEED.render2(g,"animation speed",new Color(0,150,0));
	    if(TR.halt==true) GO.render(g,"go",12,4,new Color(0,150,0));
	    if(TR.halt==false) STOP.render(g,"stop",12,4,new Color(200,0,0));
    }

    
    
    public double getStepsize() {
	return STEP.getValue();
    }

    public void setTracer(Point X) {
	 SPEED.process(X);
	 if(GO.inside(X)==1) doGo();
	 if(STOP.inside(X)==1) TR.halt=true;
    }

    public void doGo() {
	if(TR.halt==false) return;
	TR=new Tracer(this.M);
	new Thread(TR).start();
    }

    
    public void mouseClicked(MouseEvent e) {
        MouseData J=MouseData.process(e);
	int test=0;
	test=VIEW.toggle(J.X);
	if(test==20) DocControls.viewLock(M.D);
	test=DISPLAY.process(J.X,CS.C);
	if(test==20) DocControls.solDisplay(M.D);
	test=DISPLAY2.process(J.X,CS.C);
	if(test==20) DocControls.lieDisplay(M.D);
	test=ACTION.switchMode(J.X);
	if(test==20) DocControls.lockDisplay(M.D);
	STEP.configure(J.X);
	test=SYMM.switchMode(J.X);
	if(test==20) DocControls.symmetry(M.D);
	POP.process(J.X);
	test=CS.process(e);
	if(test==20) DocControls.colorSelector(M.D);
	if(INFO.inside(J.X)==1) DocControls.step(M.D);
	if(INFO2.inside(J.X)==1) DocControls.tracer(M.D);
	YIN.process(J.X);
	setTracer(J.X);

	M.repaint();
    }
    
    
    public void mousePressed(MouseEvent e) {
	MouseData J=MouseData.process(e);
	STEP.activate(J.X);
	repaint();
    }
    
     public void mouseReleased(MouseEvent e) {}
     public void mouseEntered(MouseEvent e) {}
     public void mouseExited(MouseEvent e) {}   
     public void mouseMoved(MouseEvent e) {}    
     public void mouseDragged(MouseEvent e) {

	MouseData J=MouseData.process(e);
	STEP.configure(J.X);

	CS.process(e);
	M.repaint();
     } 

}

