import java.awt.*;
import java.math.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;


public class Curve {

    public static Path2D.Double curve(Complex[] W,int COUNT) {
	Path2D.Double gp=new Path2D.Double();
	if(COUNT==0) return null;
	for(int i=0;i<COUNT;++i) {
	    if(i==0) gp.moveTo(W[i].x,W[i].y);
	    if(i!=0) gp.lineTo(W[i].x,W[i].y);
	}
	return gp;
    }


    /**This computes a Gaussian smoothing of the curve where t is the
       sharpness of the Gaussian. 
       t=infty means delta function.
       N is the number of line segments in the output path*/

    public static Complex[] gaussianAverage(Complex[] Z,int COUNT,double t,int N) {
	Complex[] W=new Complex[N+1];
	for(int i=0;i<N+1;++i) {
            W[i]=gaussianAverage(Z,COUNT,t,N,i);
	}
	return W;
    }

    public static Complex gaussianAverage(Complex[] Z,int COUNT,double t,int N,int k) {
	double pos=1.0*k/N;
	double WEIGHT=0;
	Complex W=new Complex(0,0);
	for(int i=0;i<COUNT;++i) {
	    double pos2=1.0*i/COUNT;
	    pos2=pos2-pos;
	    double weight=Math.pow(2,-t*pos2*pos2);
	    WEIGHT=WEIGHT+weight;
	    Complex w=Complex.times(Z[i],new Complex(weight,0));
	    W=Complex.plus(W,w);
	}
	W=Complex.times(W,new Complex(1.0/WEIGHT,0));
	return W;
    }



    public static Complex[] gaussianFunction(Complex[] Z,int COUNT,double t) {
	Complex[] W=new Complex[COUNT];
	for(int i=0;i<COUNT;++i) W[i]=gaussianFunction(Z,COUNT,t,i);
	return W;
    }

    public static Complex gaussianFunction(Complex[] Z,int COUNT,double t,int k) {
	double pos=1.0*k/COUNT;
	double WEIGHT=0;
	Complex W=new Complex(0,0);
	for(int i=0;i<COUNT;++i) {
	    double pos2=1.0*i/COUNT;
	    pos2=pos2-pos;
	    double weight=Math.pow(2,-t*pos2*pos2);
	    WEIGHT=WEIGHT+weight;
	    Complex w=Complex.times(Z[i],new Complex(weight,0));
	    W=Complex.plus(W,w);
	}
	W=Complex.times(W,new Complex(1.0/WEIGHT,0));
	W.x=1.0*k/COUNT;
	return W;
    }



    /**This finds the point Z[index] closest to the point w.*/

    public static int nearestIndex(Complex[] Z,int COUNT,Complex w) {
	int index=0;
	double min=100000;
	for(int i=0;i<COUNT;++i) {
	    double test=Complex.dist(Z[i],w);
	    if(test<min) {
		index=i;
		min=test;
	    }
	}
	return index;
    }


}

