import java.awt.event.*;
import java.awt.*;
import java.math.*;
import java.awt.geom.*;


public class JacobiComputer {

    public static Vector[] jacobi(double step1,double step2,double t) {
	Vector U0=makeVector(t);
	Vector U1=makeVector(t+step2);
	double[] data0=GeodesicMaker.periodData(step1,U0);
	double[] data1=GeodesicMaker.periodData(step1,U1);
	double length0=data0[0];
	double length1=data1[0];
	double step3=step2*(length1/length0);
	Vector[] V0=GeodesicMaker.geodesic(step2,length0,U0);
	Vector[] V1=GeodesicMaker.geodesic(step3,length1,U1);
	int num=V0.length;
	if(num>V1.length) num=V1.length;
	Vector[] W=new Vector[num];
	for(int i=0;i<num;++i) {
            W[i]=Vector.minus(V1[i],V0[i]);
	    W[i]=W[i].scale(1/step2);
	    W[i]=differentialTranslate(V0[i],W[i]);
	}

	System.out.println("vectors ");
	U0.print();
	U1.print();
	Vector UU=Vector.minus(U1,U0);
	UU.print();
	System.out.println("last ");
	W[W.length-1].print();
	double ratio=W[W.length-1].x[0]/U0.x[2];
	System.out.println("ratio "+ratio);
	return W;
    }


    public static Vector makeVector(double t) {
	Vector U=new Vector(0,0,t);
	double x=Math.sqrt(1-t*t)/Math.sqrt(2);
	U.x[0]=x;
	U.x[1]=x;
	return U;
    }



    public static Vector differentialTranslate(Vector V,Vector W) {
	double x=V.x[0];
	double y=V.x[1];
	double z=V.x[2];
	double a=W.x[0];
	double b=W.x[1];
	double c=W.x[2];
	double aa=Math.exp(-z)*a;
	double bb=Math.exp(+z)*b;
	double cc=c;
	Vector Z=new Vector(aa,bb,cc);
	return Z;
    }

    
}
