import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.math.*;


public class LieCanvas extends ScaleCanvas implements MouseListener, MouseMotionListener, KeyListener {
    Manager M;
    Vector V;
    Point JX;
    boolean drag;
    ListenSquare CONTROL;
    EnhancedSlider PITCH,YAW;
    Vector[][] POINT0=new Vector[20][1000];
    Vector[][] POINT1=new Vector[20][1000];

     public LieCanvas() {
	 addMouseListener(this);
	 addMouseMotionListener(this);
	 addKeyListener(this);
	 setScales(300,340,190,-190);
	 V=new Vector(.6,0,.8);
	 CONTROL=new ListenSquare(0,0,600,90);
	 YAW=new EnhancedSlider(0,0,530,27,265,new Color(0,0,255),"yaw");
	 PITCH=new EnhancedSlider(0,30,530,27,265,new Color(0,160,0),"pitch");
	 YAW.forceValue(.125);
	 PITCH.forceValue(.5);
	 drag=false;
     }

   public void paint(Graphics g2) {
      Graphics2D g=(Graphics2D) g2;
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
      drawBG(g);
      drawVector(g);
      drawLoops(g);
      drawRadials(g);
      drawControls(g);
   }

    public void drawBG(Graphics2D g) {
	g.setColor(new Color(120,130,140));
        g.fillRect(0,0,getWidth(),getHeight());
	fillPoint(g,new Complex(0,0),1,new Color(0,0,0),2000);
    }

    public void drawLoops(Graphics2D g) {
	int N=M.B.NUM[1].getInteger();
	double crit=M.V.criticalValue(M.V.getRadius());
	Path2D.Double gp=new Path2D.Double();
	boolean active=false;
	boolean alive=false;
	for(int q1=-1;q1<=1;q1=q1+2) {
	for(int q2=-1;q2<=1;q2=q2+2) {
	for(int j=0;j<N;++j) {
	    active=false;
	    alive=false;
	    gp.reset();
	    double r0=1.0*j/(N-1);
	    double r=(1-r0)*crit+r0;
	    int step=200;
	    if(r>.95) step=5000;
	    for(int i=0;i<step;++i) {
		double theta=2*Math.PI*i/(step-1);
	       double x=r*Math.cos(theta);
	       double y=r*Math.sin(theta);
	       Vector U=Vector.chartToSphere(x,y); 
	       U.x[0]=U.x[0]*q1;
	       U.x[1]=U.x[1]*q2;
	       if(isVisible(U)==false) active=false;
	       
	       if(isVisible(U)==true) {
		      double[] s=view(U);
	              if(active==false) gp.moveTo(s[0],s[1]);
	              if(active==true) gp.lineTo(s[0],s[1]);
		      active=true;
		      alive=true;
	       }
	    }
	    if(alive==true) {
	        gp=transform(gp);
	        g.setColor(new Color(40,50,60));
	        if(q1+q2==2) g.setColor(M.C.DISPLAY.M[1].C);
		g.setStroke(new BasicStroke((float)(1.5)));
	         g.draw(gp);
	    }
	    
	}}}
	g.setStroke(new BasicStroke(1));
    }

    public void drawRadials(Graphics2D g) {
	int N=1;
	try{
	   N=M.B.NUM[2].getInteger();
	}
	catch(Exception e) {}
	double crit=M.V.criticalValue(M.V.getRadius());
	Path2D.Double gp=new Path2D.Double();
	boolean active=false;
	boolean alive=false;
	for(int q1=-1;q1<=1;q1=q1+2) {
	for(int q2=-1;q2<=1;q2=q2+2) {
	for(int j=0;j<N;++j) {
	    active=false;
	    alive=false;
	    gp.reset();
	    double theta=2*Math.PI*1.0*j/(N-1);
	    int step=100;
	    for(int i=0;i<step;++i) {
		double r=1.0*i/(step-1);
                r=(1-r)*crit+r;
	        double x=r*Math.cos(theta);
	        double y=r*Math.sin(theta);
	       Vector U=Vector.chartToSphere(x,y); 
	       U.x[0]=U.x[0]*q1;
	       U.x[1]=U.x[1]*q2;
	       if(isVisible(U)==false) active=false;
	       
	       if(isVisible(U)==true) {
		      double[] s=view(U);
	              if(active==false) gp.moveTo(s[0],s[1]);
	              if(active==true) gp.lineTo(s[0],s[1]);
		      active=true;
		      alive=true;
	       }
	    }
	    if(alive==true) {
	        gp=transform(gp);
	        g.setColor(new Color(70,80,90));
		if(q1+q2==2) g.setColor(M.C.DISPLAY.M[2].C);
		g.setStroke(new BasicStroke((float)(1.5)));
		g.setStroke(new BasicStroke((float)(1.5)));
	         g.draw(gp);
	    }
	    
	}}}
	g.setStroke(new BasicStroke(1));
    }

    
    
    public void drawVector(Graphics2D g) {
	double[] A=view(V);
	Path2D.Double gp=new Path2D.Double();
	gp.moveTo(A[0],A[1]);
	gp.lineTo(A[0],A[1]);
	g.setColor(Color.white);
	g.setStroke(new BasicStroke(4));
	gp=transform(gp);
	g.draw(gp);
	g.setStroke(new BasicStroke(1));
    }

    public void drawControls(Graphics2D g) {
	PITCH.render(g);
	YAW.render(g);
    }


    public Vector pitch(Vector V) {
  	  double t=PITCH.getValue();
	  t=2*Math.PI*t;
	  return Vector.pitch(t,V);
    }

    public Vector yaw(Vector V) {
	double t=YAW.getValue();
	t=2*Math.PI*t;
	return Vector.yaw(t,V);
    }

    public Vector pitchInverse(Vector V) {
  	  double t=PITCH.getValue();
	  t=2*Math.PI*t;
	  return Vector.pitch(-t,V);
    }

    public Vector yawInverse(Vector V) {
	double t=YAW.getValue();
	t=2*Math.PI*t;
	return Vector.yaw(-t,V);
    }

    public void setUnitVector() {
	setUnitVector(SOURCE);
    }

    public void setUnitVector(Complex z) {
	double d=z.norm();
	if(d>1) z=z.unit();
	d=z.norm();
	V.x[0]=z.x;
	V.x[2]=z.y;
	V.x[1]=Math.sqrt(1-d*d);
	V=yawInverse(V);
	V=pitchInverse(V);
	M.V.V=new Vector(V);
	double[] A=V.sphereToChart();
	M.V.SOURCE=new Complex(A[0],A[1]);
    }

    
    public Vector normal() {
	Vector N=new Vector(0,1,0);
	return yaw(pitch(N));
    }

    public boolean isVisible(Vector V) {
	Vector N=new Vector(0,1,0);
	double[] p=view(V);
	Vector P=new Vector(p[0],p[2],p[1]);
	double test=Vector.dot(P,N);
	if(test>0) return true;
	return false;
    }
    


    public double[] view(Vector V) {
	Vector W=pitch(V);
	W=yaw(W);
	double[] A={W.x[0],W.x[2],W.x[1]};
	return A;
    }

    public double[] viewInverse(Vector V) {
	Vector W=yawInverse(V);
	W=pitchInverse(W);
	double[] A={W.x[0],W.x[2]};
	return A;
    }

    public void mousePressed(MouseEvent e) { 
	MouseData J=MouseData.process(e);
        PITCH.activate(J.X);
	YAW.activate(J.X);
	drag=true;
    }


    public void mouseClicked(MouseEvent e) { 
	MouseData J=MouseData.process(e);
	doMouseClick(J.mode);
    }

    public void doMouseClick(int mode) { 
	if(CONTROL.inside(JX)==1) {
	    PITCH.configure(JX);
	    YAW.configure(JX);
	    M.repaint();
	    return;
	}
        if(mode==1)  scaleUp(JX,0);
        if(mode==3)  scaleUp(JX,1);
	if(mode==2) SOURCE=unTransform(JX);
	setUnitVector();
	M.repaint();
    }
     public void mouseReleased(MouseEvent e) {	
	 drag=false; 
     }

     public void mouseEntered(MouseEvent e) {}
     public void mouseExited(MouseEvent e) {
	 requestFocus();
     }  

     public void mouseMoved(MouseEvent e) {
	MouseData J=MouseData.process(e);
	JX=J.X;
	if(drag==true) {
	    SOURCE=unTransform(J.X);
	}
     }

     public void mouseDragged(MouseEvent e) {

	MouseData J=MouseData.process(e);
	if(CONTROL.inside(J.X)==1) {
	    PITCH.configure(J.X,true);
	    YAW.configure(J.X,true);
  	    M.repaint();
	    return;

	}
	if(J.mode==2) SOURCE=unTransform(J.X);

	setUnitVector();
	M.repaint();
	 
     }

  public void keyTyped(KeyEvent e) {
      char ch=e.getKeyChar();
      if(ch=='z') doMouseClick(1);
      if(ch=='x') doMouseClick(2);
      if(ch=='c') doMouseClick(3);      
     }
    public void keyPressed(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {
    }

}

