import java.awt.event.*;
import java.awt.*;

/*This class does the basic arithmetic
  of vectors */


public class Vector {
    double[] x=new double[3];
    int vertex;  //names the vertex of an auxiliary polygon

    public Vector(){}

    public Vector(double xx,double yy,double zz) {
	x[0]=xx;
	x[1]=yy;
	x[2]=zz;
    }

    public Vector(double[] xx) {
	x[0]=xx[0];
	x[1]=xx[1];
	x[2]=xx[2];
    }

    public Vector(Vector X) {
	x[0]=X.x[0];
	x[1]=X.x[1];
	x[2]=X.x[2];
    }
    
    public static Vector plus(Vector V,Vector W) {
	return(new Vector(V.x[0]+W.x[0],V.x[1]+W.x[1],V.x[2]+W.x[2]));
    }

    public static Vector minus(Vector V,Vector W) {
	return(new Vector(V.x[0]-W.x[0],V.x[1]-W.x[1],V.x[2]-W.x[2]));
    }

    public Vector scale(double d) {
	Vector X=new Vector(this);
	for(int i=0;i<3;++i) X.x[i]=X.x[i]*d;
	return(X);
    }

    public Vector unit() {
	double d=this.norm();
	return this.scale(1/d);
    }
    
    public static double dist(Vector V1,Vector V2) {
	Vector X=Vector.minus(V1,V2);
	return X.norm();
    }

    public static double  dot(Vector v,Vector w) {
	return(v.x[0]*w.x[0]+v.x[1]*w.x[1]+v.x[2]*w.x[2]);
    }


    public double norm() {
	return(Math.sqrt(dot(this,this)));
    }

    public Vector proj(Vector W) {
	Vector V1=this.unit();
	double d=dot(V1,W);
	Vector V2=V1.scale(d);
	Vector X=minus(W,V2);
	return(X);
    }


    public static Vector cross(Vector v,Vector w) {
	Vector X=new Vector();
	X.x[0]=v.x[1]*w.x[2]-w.x[1]*v.x[2];
	X.x[1]=v.x[2]*w.x[0]-w.x[2]*v.x[0];
	X.x[2]=v.x[0]*w.x[1]-w.x[0]*v.x[1];
	return(X);
    }

    public static Vector findCross(Vector v1,Vector v2,Vector v3,Vector v4) {
	Vector v5=cross(v1,v2);
	Vector v6=cross(v3,v4);
	Vector v7=cross(v5,v6);
	return(v7);
    }



    public static Vector cycle(Vector V) {
	return new Vector(V.x[2],V.x[0],V.x[1]);
    }


    public static Vector pitch(double tt,Vector V0) {
	Vector V=cycle(V0);
	V=pitch0(tt,V);
	V=cycle(cycle(V));
	return V;
    }
    
    public static Vector pitch0(double tt,Vector V) {
          double x0=V.x[0];
	  double y0=V.x[1];
	  double z0=V.x[2];
	  double c=Math.cos(tt);
	  double s=Math.sin(tt);
	  double x1=x0*c+z0*s;
	  double y1=y0;
	  double z1=-x0*s+z0*c;
	  Vector W=new Vector(x1,y1,z1);
	  return W;
    }


    public static Vector yaw(double t,Vector V) {
	double c=Math.cos(t);
	double s=Math.sin(t);
        double x=V.x[0];
	double y=V.x[1];
        double xx=x*c+y*s;
        double yy=-x*s+y*c;
	double z=V.x[2];
	Vector W=new Vector(xx,yy,z);
	return W;
    }



    /**These are coordinates for the coordinate chart*/
    
    public static Vector chartToSphere(double s0,double t0) {

	double[] m1={s0,t0};
	m1=K2P(m1);
	double s=m1[0];
	double t=m1[1];
	
	double x=s;
	double z=t;
	double y=1-x*x-z*z;
	if(y<0) y=0;
	y=Math.sqrt(y);
	Complex A=new Complex(x,y);
	A=Complex.sqrt(A);
	double z2=1-A.x*A.x-A.y*A.y;
	if(z2<0) z2=0;
	z2=Math.sqrt(z2);
	if(t<0) z2=-z2;
	Vector U=new Vector(A.x,A.y,z2);
	return U;
    }

    public double[] sphereToChart() {
	Complex z=new Complex(x[0],x[1]);
	z=Complex.times(z,z);
	double x2=1-z.x*z.x-z.y*z.y;
	if(x2<0) x2=0;
	x2=Math.sqrt(x2);
	if(x[2]<0) x2=-x2;
	double[] A={z.x,x2};
	A=P2K(A);
        return A;
    }


    public static double[] K2P(double[] a) {
	double x=a[0];
	double y=a[1];
	double xx=2*x/(1+x*x+y*y);
	double yy=2*y/(1+x*x+y*y);
	double[] b={xx,yy};
	return b;
    }

    public static double[] P2K(double[] a) {
	double x=a[0];
	double y=a[1];
	double r=1-x*x-y*y;
	if(r<0) r=0;
	double xx=x/(1+Math.sqrt(r));
	double yy=y/(1+Math.sqrt(r));
	double[] b={xx,yy};
	return b;
    }

    /**end coords for coordinate chart*/

    
    public static Vector spherePoint(double s,double t) {
	double x=Math.cos(2*Math.PI*s);
	double y=Math.sin(2*Math.PI*s);
	double phi=-Math.cos(Math.PI*t);
	double rho=Math.sqrt(1-phi*phi);
	x=x*rho;
	y=y*rho;
	Vector U=new Vector(x,y,phi);
	return U.unit();
    }



    public void print() {
	System.out.println(x[0]+" "+x[1]+" "+x[2]);
    }


}




