import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.math.*;


public class VectorCanvas extends ScaleCanvas implements MouseListener, MouseMotionListener, KeyListener {
    Manager M;
    Vector V; //this is the current vector.
    Point JX;
    boolean drag;
    ListenSquare CONTROL;
    EnhancedSlider RADIUS1;
    EnhancedSlider PITCH,YAW;
    Lever RADIUS2;
    Complex[] BOX=new Complex[2];
    double[] ANN=new double[2];
    Path2D.Double YINPATH,FLOWLINE;

    Complex[] PLOT=new Complex[500000];
    Color [] COL=new Color[500000];
    int COUNT=0;

     public VectorCanvas() {
	 addMouseListener(this);
	 addMouseMotionListener(this);
	 addKeyListener(this);
	 setScales(300,485,230,-230);
	 V=new Vector(.6,0,.8);
	 CONTROL=new ListenSquare(0,0,599,225);
	 RADIUS1=new EnhancedSlider(1,0,529,27,529,new Color(0,200,200),"radius (R) coarse");
	 RADIUS2=new Lever(5,32,5,39);
	 setView();
	 drag=false;
	 BOX[0]=new Complex(-1,-1);
	 BOX[1]=new Complex(1,1);
	 ANN[0]=0;
	 ANN[1]=1;

	 YINPATH=new Path2D.Double();

     }


    public void setView() {
         YAW  =new EnhancedSlider(1,165,529,27,265,new Color(200,0,200),"yaw");
	 PITCH=new EnhancedSlider(1,195,529,27,265,new Color(200,0,200),"pitch");
	 YAW.forceValue(.125);
	 PITCH.forceValue(.5);
	 YAW.POW.val=1;
	 PITCH.POW.val=1;
    }
    

   public void paint(Graphics g2) {
      Graphics2D g=(Graphics2D) g2;
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
      drawBG(g);
      drawFoliation(0,g);
      drawFoliation(1,g);
      drawFoliation(2,g);
      drawVector(g);
      drawYin(g);
      drawFlowline(g);
      drawCritical(g);
      drawControls(g);
      drawCoords(g);
   }
    
    public void drawBG(Graphics2D g) {
	g.setColor(new Color(70,90,110));
	g.fillRect(0,0,this.getWidth(),this.getHeight());
	Color C=M.C.DISPLAY2.M[0].C;
	fillPoint(g,new Complex(0,0),1,C,512);
    }


    public static double criticalValue(double length) {
	double val0=EllipticIntegrals.periodInverse(length); //critical level set F-value
	val0=Math.sqrt(val0);
	Vector CRIT=new Vector(val0,val0,Math.sqrt(1-2*val0));
	if(Double.isNaN(CRIT.x[2])==true) CRIT.x[2]=0;
	double[] A=CRIT.sphereToChart();
	double val=Math.sqrt(A[0]*A[0]+A[1]*A[1]);
	return val;
    }



    
    public void drawFoliation(int choice, Graphics2D g) {
	if(M.C.DISPLAY.L[1+choice].on==0) return;
	int N=M.B.NUM[1+choice].getInteger()+2;

	Path2D.Double gp=new Path2D.Double();
	boolean active=false;
	boolean alive=false;

	double[] LIM=getLimits();
	double lim1=LIM[0];
	double lim2=LIM[1];
	if(lim2>.99) lim2=1;

	for(int q1=-1;q1<=1;q1=q1+2) {
	for(int q2=-1;q2<=1;q2=q2+2) {
	for(int j=0;j<N;++j) {
	    active=false;
	    alive=false;
	    gp.reset();

	    Vector[] LIST=new Vector[0];
	    int step=0;

	    if(choice==0) {
	       double r0=1.0*j/(N-1);
	       double r=(1-r0)*lim1 + r0*lim2;
	       step=500;
	       LIST=LevelCurve.curveCheap(step,r);
	    }

	    if(choice==1) {
               double theta=.5*Math.PI*1.0*j/(N-1);
	       step=200;
	       LIST=LevelCurve.curveLongitude(step,theta);
	    }

	    if(choice==2) {
               double theta=.5*Math.PI*1.0*j/(N-1);
	       step=200;
	       LIST=LevelCurve.curveLattitude(step,theta);
	    }
	    
	    LIST=reflect(q1,q2,LIST);
	    gp=toPath(LIST);
	    
	    if(gp!=null) {
	        gp=transform(gp);
	        g.setColor(new Color(40,50,60));
	        if(q1+q2==2) g.setColor(M.C.DISPLAY.M[1+choice].C);
		g.setStroke(new BasicStroke((float)(1.5)));
	         g.draw(gp);
	    }
	    
	}}}
	g.setStroke(new BasicStroke(1));
    }


    Path2D.Double toPath(Vector[] LIST) {
	boolean active=false;
	Path2D.Double gp=new Path2D.Double();
	double step=LIST.length;
        for(int i=0;i<step;++i) {
  	   Vector U=new Vector(LIST[i]);
	   boolean viz=isVisible(U);
	   boolean large=isOutsideLimits(U);
           if(large==true) viz=false;
	   if(viz==false) active=false;
	       
	   if(viz==true) {
	     double[] s=view(U);
	     if(active==false) gp.moveTo(s[0],s[1]);
	     if(active==true) gp.lineTo(s[0],s[1]);
	     active=true;
	   }
	}
	return gp;
    }


    public Vector[] reflect(int q1,int q2,Vector[] LIST0) {
	Vector[] LIST1=new Vector[LIST0.length];
	for(int i=0;i<LIST1.length;++i) {
	    LIST1[i]=new Vector(q1*LIST0[i].x[0],q2*LIST0[i].x[1],LIST0[i].x[2]);
	}
	return LIST1;
    }
    

    public boolean isOutsideLimits(Vector U) {
	double[] A=getLimits();
	double[] B=U.sphereToChart();
	double r=Math.sqrt(B[0]*B[0]+B[1]*B[1]);
	if(r<A[0]-.00001) return true;
	if(r>A[1]+.001) return true;
	return false;
    }

    public double[] getLimits() {
	double lim1=0;
	double lim2=1;
	double length=getRadius();
	double crit=VectorCanvas.criticalValue(length);
	lim1=Math.min(ANN[0],ANN[1]);
	lim2=Math.max(ANN[0],ANN[1]);
	lim1=Math.max(crit,ANN[0]);
	lim2=Math.min(.999,ANN[1]);
	double[] A={lim1,lim2};
	return A;
    }


    

    
    
    public double getRadius() {
	double e1=RADIUS1.getValue();
	double v=RADIUS2.val;
	v=Math.pow(2,v);
	e1=Math.floor(v*e1)/v;
	return e1;
    }

    public Path2D.Double getBox(Complex[] box) {
	Path2D.Double gp=new Path2D.Double();
	gp.moveTo(box[0].x,box[0].y);
	gp.lineTo(box[1].x,box[0].y);
	gp.lineTo(box[1].x,box[1].y);
	gp.lineTo(box[0].x,box[1].y);
	gp.closePath();
	return gp;
    }



    public void drawControls(Graphics2D g) {
	CONTROL.render(g,new Color(20,30,50));
	RADIUS1.render(g);
	PITCH.render(g);
	YAW.render(g);
	RADIUS2.render2(g,"",new Color(0,200,200));

        g.setFont(new Font("Helvetica",Font.PLAIN,14));
	g.setColor(Color.yellow);
  	g.drawString("key 1: Ann edge 1",460,120);
	g.drawString("key 2: Ann edge 2",460,138);
	g.drawString("key 3: Ann reset",460,156);
	g.drawString("dyadic rounding",480,42);

    }


    
    public void drawVector(Graphics2D g) {
	Path2D.Double gp=new Path2D.Double();
	double[] s=view(V);
	gp.moveTo(s[0],s[1]);
	gp.lineTo(s[0],s[1]);
	g.setStroke(new BasicStroke(4));
	boolean test=isVisible(V);
	g.setColor(Color.white);
	if(test==false) g.setColor(new Color(50,50,50));
	gp=transform(gp);
	g.draw(gp);
	g.setStroke(new BasicStroke(1));
    }




    /***coordinate drawing system*/
    public void drawCoords(Graphics2D g) {
	g.setFont(new Font("Helvetica",Font.PLAIN,17));
	g.setColor(new Color(200,200,255));
	
	for(int i=0;i<3;++i) {
	    if(Math.abs(V.x[2])<.000000000001) V.x[2]=0;
	    Double D=new Double(V.x[i]);
	    g.drawString(D.toString(),5,90+21*i);
	}

	Double D=new Double(getRadius());
	g.setColor(new Color(0,255,255));
	g.drawString("R "+D.toString(),5,65);

	g.setFont(new Font("Helvetica",Font.PLAIN,15));
	D=new Double(V.x[1]/V.x[0]);
	g.setColor(new Color(140,255,255));
	g.drawString("y/x "+D.toString(),5,157);

	
	
	if(M.C.DISPLAY2.L[2].on==0) return;
	g.setColor(M.C.DISPLAY2.M[2].C);

	g.setFont(new Font("Helvetica",Font.PLAIN,15));
	double step=M.C.STEP.getValue();
	
	double[] data=GeodesicMaker.periodData(step,V);
	Double[] Data=new Double[6];
	for(int i=0;i<6;++i) Data[i]=new Double(data[i]);
	String[] S={"period","x hol","y hol","z hol","hol invt.","xhol/yhol"};
	for(int i=0;i<6;++i) {
	    if(i==5) g.setColor(new Color(140,255,255));
	    g.drawString(S[i]+" "+Data[i].toString(),220,62+19*i);
	}
    }
    /***end coordinate drawing system*/



    /**Flowline drawing system*/
    
    public void computeFlow() {
	if(M.C.DISPLAY2.L[1].on==0) return;
	FLOWLINE=new Path2D.Double();
	double step=M.C.STEP.getValue();
	double length=getRadius();
	Vector[] LIST=GeodesicMaker.flow(step,length,V);
	FLOWLINE=toPath(LIST);
    }

    public void drawFlowline(Graphics2D g) {
	if(M.C.DISPLAY2.L[1].on==0) return;
	if(FLOWLINE==null) return;
	Path2D.Double gp=new Path2D.Double(FLOWLINE);
	gp=transform(gp);
	g.setStroke(new BasicStroke(2));
	g.setColor(M.C.DISPLAY2.M[1].C);
	g.draw(gp);
	g.setStroke(new BasicStroke(1));
    }
    /***end flowline system*/
    


    /**critical level*/
    public void drawCritical(Graphics2D g) {
	double[] A=getLimits();
	if(A[1]>.995) A[1]=1;
	for(int i=-1;i<=1;i=i+2) {
	for(int j=-1;j<=1;j=j+2) {
	    for(int k=0;k<2;++k) {
	    Vector[] LIST=LevelCurve.curveCheap(2000,A[k]);
	    LIST=reflect(i,j,LIST);
	    Path2D.Double gp=toPath(LIST);
	    g.setColor(new Color(130,140,150));
	    if(i+j==2) g.setColor(M.C.DISPLAY.M[4+k].C);
	    gp=transform(gp);
	    g.draw(gp);
	    }}}
    }




    
    
    /**The Yin Yang curve.  This is the set of points corresponding to points in the
       Sol spheres where the tangent plane contains a vertical vector*/
    public void makeYin() {

    }


    public Vector[] yinCurve() {
	Vector[] LIST=yinCurve0();
	int N=LIST.length;
	Vector[] LIST2=new Vector[2*N];
	for(int j=0;j<N;++j) {
	    int i=N-1-j;
	    LIST2[j]=new Vector(LIST[i]);
	    LIST2[j+N]=new Vector(LIST[j].x[1],LIST[j].x[0],-LIST[j].x[2]);
	}
	return LIST2;
    }


    public Vector[] yinCurve1() {
	Vector[] LIST=yinCurve0();
	int N=LIST.length;
	Vector[] LIST2=new Vector[N];
	for(int j=0;j<N;++j) {
	    LIST2[j]=new Vector(LIST[j].x[1],LIST[j].x[0],-LIST[j].x[2]);
	}
	return LIST2;
    }

    public Vector[] yinCurve0() {
	double step=M.C.STEP.getValue();
	double crit=criticalValue(getRadius());
	double R=getRadius();
	double lim0=ANN[0];
	double lim1=ANN[1];
	if(lim0<crit) lim0=crit;
	int N=(int)(10*Math.pow(2,M.C.YIN.val));
	Vector[] LIST=new Vector[N];
	
	for(int j=0;j<N;++j) {
	    double r0=1.0*j/(N-1);
	    r0=Math.sqrt(r0);
	   double r=(1-r0)*lim0+r0*lim1;
	   Vector START=Vector.chartToSphere(r,0);
           LIST[j]=GeodesicMaker.flowEndpointBackwards(step,R,START);
	}
	return LIST;
    }

    public void drawYin(Graphics2D g) {
	if(M.C.DISPLAY2.L[3].on==0) return;
	Vector[] LIST=yinCurve();
	YINPATH=toPath(LIST);
	Path2D.Double gp1=transform(YINPATH);
	g.setColor(M.C.DISPLAY2.M[3].C);
	g.draw(gp1);
    }
    /**End yin yang plots*/
    




    

    /**coordinate lock system: various coord restrictions*/
    public void doLocks() {
	int t=M.C.ACTION.mode;

	if(t==1) {
	    Vector U=new Vector(V);
	    U.x[2]=0;
	    V=U.unit();
	    double[] A=view(V);
	    SOURCE=new Complex(A[0],A[1]);
	}

	if(t==2) {
	    double val=criticalValue(getRadius());
            double[] A=V.sphereToChart();
	    Complex z=new Complex(A[0],A[1]);
	    z=z.unit();
	    z=Complex.times(z,new Complex(val,0));
	    V=Vector.chartToSphere(z.x,z.y);
	    A=view(V);
	    SOURCE=new Complex(A[0],A[1]);
	}

	if(t==3) {
	    double crit=Math.PI*Math.sqrt(2);
	    RADIUS1.forceValue(crit);
	    RADIUS2.val=38;
	}

	
	if(t==4) {
	    double per=EllipticIntegrals.period(V.x[0]*V.x[1]);
	    RADIUS1.forceValue(per);
	    RADIUS2.val=38;
	}
    }
    /**end coord lock system*/

    


    public void mousePressed(MouseEvent e) { 
	MouseData J=MouseData.process(e);
	RADIUS1.activate(J.X);
	PITCH.activate(J.X);
	YAW.activate(J.X);
    }

    public void doConfigure(Point X,boolean arrows) {
	double t1=getRadius();
        RADIUS1.configure(X,arrows);
	RADIUS2.process(X);
        PITCH.configure(X);
        YAW.configure(X);


	if(M.C.VIEW.L[0].on==1) {
	   M.B.PITCH.forceValue(PITCH.getValue());
	   M.B.YAW.forceValue(YAW.getValue());
	}
	
	if(M.C.DISPLAY2.L[3].on==1) makeYin();
	computeFlow();
	double t2=getRadius();
    }


    public void mouseClicked(MouseEvent e) { 
	MouseData J=MouseData.process(e);
	doMouseClick(J.mode);
    }

    public void doMouseClick(int mode) { 
	if(CONTROL.inside(JX)==1) {
	    doConfigure(JX,false);
	    doLocks();
	    M.repaint();
	 
	    return;
	}
        if(mode==1)  scaleUp(JX,0);
        if(mode==3)  scaleUp(JX,1);
	if(mode==2)  SOURCE=unTransform(JX);
	setUnitVector();
	M.repaint();
    }

    


     public void mouseReleased(MouseEvent e) {	
     }

     public void mouseEntered(MouseEvent e) {
	 requestFocus();
     }
     public void mouseExited(MouseEvent e) {
     }  

     public void mouseMoved(MouseEvent e) {
	MouseData J=MouseData.process(e);
	JX=new Point(J.X);
	if(CONTROL.inside(JX)==1) return;

	if(drag==true) {
	    SOURCE=unTransform(JX);
	    setUnitVector();
	}
	M.repaint();
     }

     public void mouseDragged(MouseEvent e) {
	MouseData J=MouseData.process(e);
	if(CONTROL.inside(J.X)==1) {
	    doConfigure(J.X,true);
  	    M.repaint();
	    return;

	}
	if(J.mode==2) {
            SOURCE=unTransform(J.X);
	}
	setUnitVector();
	M.repaint();
     }


    public void setBounds(char ch) {
	double[] A=V.sphereToChart();
	double r=A[0]*A[0]+A[1]*A[1];
	r=Math.sqrt(r);
	if(ch=='1') ANN[0]=r;
	if(ch=='2') ANN[1]=r;
        if(ch=='3') {
	    ANN[0]=0;
	    ANN[1]=1;
	}
	if(ANN[0]>1) ANN[0]=1;
	if(ANN[1]>1) ANN[1]=1;
	if(ANN[0]>ANN[1]) {
	    double temp=ANN[0];
	    ANN[0]=ANN[1];
	    ANN[1]=temp;
	}
    }


    
  public void keyTyped(KeyEvent e) {
      char ch=e.getKeyChar();
      if(ch=='z') doMouseClick(1);
      if(ch=='x') doMouseClick(2);
      if(ch=='c') doMouseClick(3);  
      setBounds(ch);
      M.repaint();
     }

    public void keyPressed(KeyEvent e) {
	drag=true;
	M.repaint();
    }

    public void keyReleased(KeyEvent e) {
	drag=false;
	M.repaint();
    }





    public Vector pitch(Vector V) {
  	  double t=PITCH.getValue();
	  t=2*Math.PI*t;
	  return Vector.pitch(t,V);
    }

    public Vector yaw(Vector V) {
	double t=YAW.getValue();
	t=2*Math.PI*t;
	return Vector.yaw(t,V);
    }

    public Vector pitchInverse(Vector V) {
  	  double t=PITCH.getValue();
	  t=2*Math.PI*t;
	  return Vector.pitch(-t,V);
    }

    public Vector yawInverse(Vector V) {
	double t=YAW.getValue();
	t=2*Math.PI*t;
	return Vector.yaw(-t,V);
    }

    public void setUnitVector() {
	setUnitVector(SOURCE);
	doLocks();
	setUnitVector(SOURCE);
	boolean compute=false;
	if(M.C.DISPLAY.L[3].on==1) compute=true;
	if(M.C.DISPLAY.L[4].on==1) compute=true;
	if(compute==true) M.B.computeGeodesic();
	computeFlow();
    }

    public void setUnitVector(Complex z) {
	double d=z.norm();
	if(d>1) z=z.unit();
	d=z.norm();
	V.x[0]=z.x;
	V.x[2]=z.y;
	V.x[1]=Math.sqrt(1-d*d);
	V=pitchInverse(V);
	V=yawInverse(V);
    }

    
    public Vector normal() {
	Vector N=new Vector(0,1,0);
	return yaw(pitch(N));
    }

    public boolean isVisible(Vector V) {
	Vector N=new Vector(0,1,0);
	double[] p=view(V);
	Vector P=new Vector(p[0],p[2],p[1]);
	double test=Vector.dot(P,N);
	if(test>0) return true;
	return false;
    }
    


    public double[] view(Vector V) {
	Vector W=yaw(V);
	W=pitch(W);
	double[] A={W.x[0],W.x[2],W.x[1]};
	return A;
    }




    
}

